from collections.abc import Callable
from pathlib import Path
from typing import Any, BinaryIO, Literal, TextIO, TypeAlias

from ._toml_rs import (
    _VERSION,
    _dumps,
    _loads,
)

__version__: str = _VERSION

TomlVersion: TypeAlias = Literal["1.0.0", "1.1.0"]

DEFAULT_TOML_VERSION = "1.0.0"


def load(
    fp: BinaryIO,
    /,
    *,
    parse_float: Callable[[str], Any] = float,
    toml_version: TomlVersion = DEFAULT_TOML_VERSION,
) -> dict[str, Any]:
    toml_bytes = fp.read()
    try:
        toml_str = toml_bytes.decode()
    except AttributeError:
        msg = "File must be opened in binary mode, e.g. use `open('foo.toml', 'rb')`"
        raise TypeError(msg) from None
    return loads(toml_str, parse_float=parse_float, toml_version=toml_version)


def loads(
    s: str,
    /,
    *,
    parse_float: Callable[[str], Any] = float,
    toml_version: TomlVersion = DEFAULT_TOML_VERSION,
) -> dict[str, Any]:
    if not isinstance(s, str):
        msg = f"Expected str object, not '{type(s).__qualname__}'"
        raise TypeError(msg)
    return _loads(s, parse_float=parse_float, toml_version=toml_version)


def dump(
    obj: Any,
    /,
    file: str | Path | TextIO,
    inline_tables: set[str] | None = None,
    *,
    pretty: bool = False,
    toml_version: TomlVersion = DEFAULT_TOML_VERSION,
) -> int:
    toml_str = _dumps(
        obj,
        inline_tables=inline_tables,
        pretty=pretty,
        toml_version=toml_version,
    )
    if isinstance(file, str):
        file = Path(file)
    if isinstance(file, Path):
        return file.write_text(toml_str, encoding="utf-8")

    return file.write(toml_str)


def dumps(
    obj: Any,
    /,
    inline_tables: set[str] | None = None,
    *,
    pretty: bool = False,
    toml_version: TomlVersion = DEFAULT_TOML_VERSION,
) -> str:
    return _dumps(
        obj,
        inline_tables=inline_tables,
        pretty=pretty,
        toml_version=toml_version,
    )


class TOMLDecodeError(ValueError):
    def __init__(self, msg: str, doc: str, pos: int, *args: Any) -> None:
        msg = msg.rstrip()
        super().__init__(msg)
        lineno = doc.count("\n", 0, pos) + 1
        if lineno == 1:  # noqa: SIM108
            colno = pos + 1
        else:
            colno = pos - doc.rindex("\n", 0, pos)
        self.msg = msg
        self.doc = doc
        self.pos = pos
        self.colno = colno
        self.lineno = lineno


class TOMLEncodeError(TypeError):
    def __init__(self, msg: str, *args: Any) -> None:
        msg = msg.rstrip()
        super().__init__(msg)
        self.msg = msg



from datetime import datetime, timedelta, timezone
from re import escape as e

import pytest
import toml_rs


@pytest.mark.parametrize(
    ("v", "pattern", "kwargs"),
    [
        (
            type("_Class", (), {}),
            r"Cannot serialize <class '.*_Class'> \(<class 'type'>\)",
            {},
        ),
        (
            {"x": lambda x: x},
            r"Cannot serialize <function <lambda> at 0x.*> \(<class 'function'>\)",
            {},
        ),
        (
            {"x": 1 + 2j},
            e("Cannot serialize (1+2j) (<class 'complex'>)"),
            {},
        ),
        (
            {"set": {1, 2, 3}},
            r"Cannot serialize {1, 2, 3} \(<class 'set'>\)",
            {},
        ),
        (
            {"valid": {"invalid": object()}},
            r"Cannot serialize <object object at 0x.*> \(<class 'object'>\)",
            {},
        ),
        (
            {42: "value"},
            e("TOML table keys must be strings, got 42 (<class 'int'>)"),
            {},
        ),
        (
            {"database": {"connection": {"host": "localhost"}}},
            e(
                "Path 'database.connectio' specified in"
                " inline_tables does not exist in the toml",
            ),
            {"inline_tables": {"database.connectio"}},
        ),
        (
            {"database": {"connection": {"host": "localhost"}, "port": 8080}},
            e("Path 'database.port' does not point to a table"),
            {"inline_tables": {"database.port"}},
        ),
    ],
)
def test_incorrect_dumps(v, pattern, kwargs):
    with pytest.raises(toml_rs.TOMLEncodeError, match=pattern):
        toml_rs.dumps(v, **kwargs)


def test_dumps():
    obj = {
        "title": "TOML Example",
        "float": float("-inf"),
        "float_2": float("+nan"),
        "owner": {
            "dob": datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(hours=-8))),
            "name": "Tom Preston-Werner",
        },
        "database": {
            "connection_max": 5000,
            "enabled": True,
            "ports": [8001, 8001, 8002],
            "server": "192.168.1.1",
        },
    }
    assert (
        toml_rs.dumps(obj)
        == """\
title = "TOML Example"
float = -inf
float_2 = nan

[owner]
dob = 1979-05-27T07:32:00-08:00
name = "Tom Preston-Werner"

[database]
connection_max = 5000
enabled = true
ports = [8001, 8001, 8002]
server = "192.168.1.1"
"""
    )


def test_dumps_inline_tables():
    obj = {
        "database": {
            "connection": {"host": "localhost", "port": 5432},
            "credentials": {"user": "admin", "password": "secret"},
        },
        "service": {
            "endpoint": "https://api.example.com",
            "parameters": {"timeout": 30, "retries": 3},
        },
    }
    dumps = toml_rs.dumps(obj)
    dumps_with_inline_tables = toml_rs.dumps(
        obj,
        inline_tables={
            "database.connection",
            "database.credentials",
            "service.parameters",
        },
    )
    dumps_with_inline_tables_2 = toml_rs.dumps(
        obj,
        inline_tables={
            "database.connection",
            "service.parameters",
        },
    )
    assert (
        dumps
        == """\
[database]

[database.connection]
host = "localhost"
port = 5432

[database.credentials]
user = "admin"
password = "secret"

[service]
endpoint = "https://api.example.com"

[service.parameters]
timeout = 30
retries = 3
"""
    )
    assert (
        dumps_with_inline_tables
        == """\
[database]
connection = { host = "localhost", port = 5432 }
credentials = { user = "admin", password = "secret" }

[service]
endpoint = "https://api.example.com"
parameters = { timeout = 30, retries = 3 }
"""
    )
    assert (
        dumps_with_inline_tables_2
        == """\
[database]
connection = { host = "localhost", port = 5432 }

[database.credentials]
user = "admin"
password = "secret"

[service]
endpoint = "https://api.example.com"
parameters = { timeout = 30, retries = 3 }
"""
    )


def test_dumps_pretty():
    obj = {
        "example": {
            "array": ["item 1", "item 2", "item 3"],
        },
        "x": [
            {"name": "foo", "value": 1},
            {"name": "bar", "value": 2},
        ],
    }
    assert (
        toml_rs.dumps(obj, pretty=False)
        == """\
x = [{ name = "foo", value = 1 }, { name = "bar", value = 2 }]

[example]
array = ["item 1", "item 2", "item 3"]
"""
    )
    assert (
        toml_rs.dumps(obj, pretty=True)
        == """\
[example]
array = [
    "item 1",
    "item 2",
    "item 3",
]

[[x]]
name = "foo"
value = 1

[[x]]
name = "bar"
value = 2
"""
    )


def test_dumps_pretty_with_inline_tables():
    obj = {
        "array": ["item 1", "item 2", "item 3"],
        "database": {
            "connection": {"host": "localhost", "port": 5432},
            "credentials": {"user": "admin", "password": "secret"},
        },
        "x": [
            {"name": "foo", "value": 1},
            {"name": "bar", "value": 2},
        ],
    }
    assert (
        toml_rs.dumps(
            obj,
            inline_tables={"database.connection", "database.credentials"},
            pretty=True,
        )
        == """\
array = [
    "item 1",
    "item 2",
    "item 3",
]
x = [
    { name = "foo", value = 1 },
    { name = "bar", value = 2 },
]

[database]
connection = { host = "localhost", port = 5432 }
credentials = { user = "admin", password = "secret" }
"""
    )



import copy
import datetime
import sys
from decimal import Decimal
from pathlib import Path
from tempfile import TemporaryDirectory

import pytest
import toml_rs as tomllib


def test_load():
    content = "one=1 \n two='two' \n arr=[]"
    expected = {"one": 1, "two": "two", "arr": []}
    with TemporaryDirectory() as tmp_dir_path:
        file_path = Path(tmp_dir_path) / "test.toml"
        file_path.write_text(content)

        with Path(file_path).open("rb") as bin_f:
            actual = tomllib.load(bin_f)
    assert actual == expected


def test_incorrect_load():
    content = "one=1"
    with TemporaryDirectory() as tmp_dir_path:
        file_path = Path(tmp_dir_path) / "test.toml"
        file_path.write_text(content)

        with (
            Path(file_path).open(encoding="utf-8") as txt_f,
            pytest.raises(TypeError) as exc,
        ):
            tomllib.load(txt_f)  # type: ignore[arg-type]
        assert str(exc.value) in (
            "File must be opened in binary mode, e.g. use `open('foo.toml', 'rb')`",
            "bytes object expected; got str",
        )


def test_parse_float():
    doc = """
          val=0.1
          biggest1=inf
          biggest2=+inf
          smallest=-inf
          notnum1=nan
          notnum2=-nan
          notnum3=+nan
          """
    obj = tomllib.loads(doc, parse_float=Decimal)
    expected = {
        "val": Decimal("0.1"),
        "biggest1": Decimal("inf"),
        "biggest2": Decimal("inf"),
        "smallest": Decimal("-inf"),
        "notnum1": Decimal("nan"),
        "notnum2": Decimal("-nan"),
        "notnum3": Decimal("nan"),
    }
    for k, expected_val in expected.items():
        actual_val = obj[k]
        assert isinstance(actual_val, Decimal)
        if actual_val.is_nan():
            assert expected_val.is_nan()
        else:
            assert actual_val == expected_val


def test_deepcopy():
    doc = """
          [bliibaa.diibaa]
          offsettime=[1979-05-27T00:32:00.999999-07:00]
          """
    obj = tomllib.loads(doc)
    obj_copy = copy.deepcopy(obj)
    assert obj_copy == obj

    expected_obj = {
        "bliibaa": {
            "diibaa": {
                "offsettime": [
                    datetime.datetime(
                        1979,
                        5,
                        27,
                        0,
                        32,
                        0,
                        999999,
                        tzinfo=datetime.timezone(datetime.timedelta(hours=-7)),
                    ),
                ],
            },
        },
    }
    assert obj_copy == expected_obj


def test_inline_array_recursion_limit():
    nest_count = 470
    recursive_array_toml = "arr = " + nest_count * "[" + nest_count * "]"
    tomllib.loads(recursive_array_toml)

    nest_count = sys.getrecursionlimit() + 2
    recursive_array_toml = "arr = " + nest_count * "[" + nest_count * "]"
    with pytest.raises(
        RecursionError,
        match=r"max recursion depth met",
    ):
        tomllib.loads(recursive_array_toml)



import json
import math
import platform
from dataclasses import dataclass
from decimal import Decimal
from pathlib import Path
from typing import Literal

import pytest
import toml_rs

from .burntsushi import convert, normalize
from .helpers import _init_only, tests_path


@dataclass(**_init_only)
class MissingFile:
    path: Path


DATA_DIR = tests_path / "data"


# Test files were taken from this commit:
# https://github.com/toml-lang/toml-test/commit/229ce2e7bb565d1704eac5f41e939870d4b1bce7
def read_toml_files_file(version: str) -> tuple:
    lines = (
        (DATA_DIR / f"files-toml-{version}")
        .read_text(encoding="utf-8", errors="ignore")
        .splitlines()
    )
    return (
        tuple(
            DATA_DIR / line.strip()
            for line in lines
            if line.strip().endswith(".toml") and line.strip().startswith("valid/")
        ),
        tuple(
            DATA_DIR / line.strip()
            for line in lines
            if line.strip().endswith(".toml") and line.strip().startswith("invalid/")
        ),
    )


VALID_TOML_V_1_0_0, INVALID_TOML_V_1_0_0 = read_toml_files_file("1.0.0")
VALID_TOML_V_1_1_0, INVALID_TOML_V_1_1_0 = read_toml_files_file("1.1.0")

VALID_V_1_0_0_EXPECTED = tuple(
    json.loads(
        (p.with_suffix(".json")).read_text(encoding="utf-8"),
    )
    for p in VALID_TOML_V_1_0_0
)
VALID_V_1_1_0_EXPECTED = tuple(
    json.loads(
        (p.with_suffix(".json")).read_text(encoding="utf-8"),
    )
    for p in VALID_TOML_V_1_1_0
)

VALID_PAIRS_1_0_0 = list(zip(VALID_TOML_V_1_0_0, VALID_V_1_0_0_EXPECTED, strict=False))
VALID_PAIRS_1_1_0 = list(zip(VALID_TOML_V_1_1_0, VALID_V_1_1_0_EXPECTED, strict=False))


@pytest.mark.parametrize(
    ("invalid", "toml_version"),
    [
        *[(pytest.param(p, "1.0.0", id=p.stem)) for p in INVALID_TOML_V_1_0_0],
        *[(pytest.param(p, "1.1.0", id=p.stem)) for p in INVALID_TOML_V_1_1_0],
    ],
)
def test_invalid_tomls(invalid: Path, toml_version: str) -> None:
    toml_bytes = invalid.read_bytes()
    try:
        toml_str = toml_bytes.decode()
    except UnicodeDecodeError:
        # Some BurntSushi tests are not valid UTF-8. Skip those.
        pytest.skip(f"Invalid UTF-8: {invalid}")
    with pytest.raises(toml_rs.TOMLDecodeError):
        toml_rs.loads(toml_str)


@pytest.mark.parametrize(
    ("valid", "expected", "toml_version"),
    [
        *[(p[0], p[1], "1.0.0") for p in VALID_PAIRS_1_0_0],
        *[(p[0], p[1], "1.1.0") for p in VALID_PAIRS_1_1_0],
    ],
    ids=[p[0].stem for p in VALID_PAIRS_1_0_0 + VALID_PAIRS_1_1_0],
)
def test_valid_tomls(
    valid: Path,
    expected: dict,
    toml_version: Literal["1.0.0", "1.1.0"],
) -> None:
    toml_str = valid.read_bytes().decode("utf-8")
    try:
        toml_str.encode(encoding="ascii")
    except UnicodeEncodeError:
        pytest.skip(f"Skipping Unicode content test: {valid.name}")
    actual = toml_rs.loads(toml_str, toml_version=toml_version)
    actual = convert(actual)
    expected_normalized = normalize(expected)
    assert actual == expected_normalized
    # Ensure that parsed toml's can be serialized back without error
    toml_rs.dumps(actual)


@pytest.mark.skipif(
    platform.python_implementation() == "PyPy",
    reason="PyPy's `Decimal` parsing hits the int string "
           "conversion digit limit for very large numbers.",
)
def test_parse_big_nums() -> None:
    big_int = 999**999
    big_float = float(f"{big_int}.{big_int}")

    t = f"x = {big_int}"
    t2 = f"x = {big_float}"
    t3 = f"x = {big_int + big_int}.{big_int}"

    assert toml_rs.loads(t, toml_version="1.1.0")["x"] == big_int
    assert math.isclose(
        toml_rs.loads(t3, toml_version="1.1.0")["x"],
        big_float,
        abs_tol=1e-9,
    )
    assert math.isclose(
        toml_rs.loads(t3, toml_version="1.1.0", parse_float=Decimal)["x"],
        Decimal(big_float),
        abs_tol=1e-9,
    )
    assert math.isclose(
        toml_rs.loads(t2, toml_version="1.1.0")["x"],
        float("inf"),
        abs_tol=1e-9,
    )



import platform
import sys
import time
from collections.abc import Callable
from importlib.metadata import version
from pathlib import Path

import altair as alt
import cpuinfo
import polars as pl
import pytomlpp
import qtoml
import rtoml
import toml
import toml_rs
import tomlkit

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib

N = 5_000


def get_lib_version(lib: str) -> str:
    if lib == "tomllib":
        return "built-in"
    return version(lib)


def benchmark(func: Callable, count: int) -> float:
    start = time.perf_counter()
    for _ in range(count):
        func()
    end = time.perf_counter()
    return end - start


def plot_benchmark(
    results: dict[str, float],
    run_type: str,
    save_path: Path,
) -> None:
    df = (
        pl
        .DataFrame({
            "parser": [f"{name} ({get_lib_version(name)})" for name in results],
            "exec_time": list(results.values()),
        })
        .sort("exec_time")
        .with_columns([
            (pl.col("exec_time") / pl.col("exec_time").min()).alias("slowdown"),
        ])
    )

    chart = (
        alt
        .Chart(df)
        .mark_bar(cornerRadiusTopLeft=6, cornerRadiusTopRight=6)
        .encode(
            x=alt.X(
                "parser:N",
                sort=None,
                title="Libraries",
                axis=alt.Axis(labelAngle=0),
            ),
            y=alt.Y(
                "exec_time:Q",
                title="Execution Time (seconds, lower=better)",
                scale=alt.Scale(domain=(0, df["exec_time"].max() * 1.1)),  # type: ignore[operator]
            ),
            color=alt.Color(
                "parser:N",
                legend=None,
                scale=alt.Scale(scheme="dark2"),
            ),
            tooltip=[
                alt.Tooltip("parser:N", title=""),
                alt.Tooltip("exec_time:Q", title="Execution Time (s)", format=".4f"),
                alt.Tooltip("slowdown:Q", title="Slowdown", format=".2f"),
            ],
        )
    )
    text = (
        chart
        .mark_text(
            align="center",
            baseline="bottom",
            dy=-2,
            fontSize=9,
            fontWeight="bold",
        )
        .transform_calculate(
            label='format(datum.exec_time, ".4f") + '
            '"s (x" + format(datum.slowdown, ".2f") + ")"',
        )
        .encode(text="label:N")
    )
    os = f"{platform.system()} {platform.release()}"
    cpu = cpuinfo.get_cpu_info()["brand_raw"]
    py = platform.python_version()
    (chart + text).properties(
        width=600,
        height=400,
        title={
            "text": f"TOML parsers benchmark ({run_type})",
            "subtitle": f"Python: {py} ({os}) | CPU: {cpu}",
        },
    ).save(save_path)


file = Path(__file__).resolve().parent
example_toml = file.parent / "tests" / "data" / "example.toml"
data = example_toml.read_bytes().decode()
fixed_data = data.replace("\r\n", "\n")

obj = tomllib.loads(example_toml.read_text())


def run(run_count: int) -> None:
    loads = {
        "toml_rs": lambda: toml_rs.loads(data, toml_version="1.1.0"),
        "rtoml": lambda: rtoml.loads(data),
        "pytomlpp": lambda: pytomlpp.loads(data),
        "tomllib": lambda: tomllib.loads(data),
        "toml": lambda: toml.loads(data),
        "qtoml": lambda: qtoml.loads(fixed_data),
        "tomlkit": lambda: tomlkit.parse(data),
    }
    dumps = {
        "toml_rs": lambda: toml_rs.dumps(obj),
        "rtoml": lambda: rtoml.dumps(obj),
        "pytomlpp": lambda: pytomlpp.dumps(obj),
        "toml": lambda: toml.dumps(obj),
        "qtoml": lambda: qtoml.dumps(obj),
        "tomlkit": lambda: tomlkit.dumps(obj),
    }
    loads = {name: benchmark(func, run_count) for name, func in loads.items()}
    dumps = {name: benchmark(func, run_count) for name, func in dumps.items()}
    plot_benchmark(loads, run_type="loads", save_path=file / "loads.svg")
    plot_benchmark(dumps, run_type="dumps", save_path=file / "dumps.svg")


if __name__ == "__main__":
    run(N)



from collections.abc import Callable
from typing import Any, Literal, TypeAlias

_VERSION: str

TomlVersion: TypeAlias = Literal["1.0.0", "1.1.0"]

def _loads(
    s: str,
    /,
    *,
    parse_float: Callable[[str], Any] = ...,
    toml_version: TomlVersion = ...,
) -> dict[str, Any]: ...

def _dumps(
    obj: Any,
    /,
    inline_tables: set[str] | None = None,
    *,
    pretty: bool = False,
    toml_version: TomlVersion = ...,
) -> str: ...



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2021 Taneli Hukkinen
# Licensed to PSF under a Contributor Agreement.

from typing import Any

import pytest
import toml_rs as tomllib


def test_line_and_col():
    # invalid mantissa
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("val=.")
    msg = str(exc.value)
    assert "line 1, column 5" in msg
    assert "invalid mantissa" in msg

    # invalid int
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("x = 0x")

    exc = exc.value

    assert exc.colno == 5
    assert exc.lineno == 1
    assert exc.pos == 4
    assert exc.doc == "x = 0x"
    assert str(exc.msg) == (
        "TOML parse error at line 1, column 5\n"
        "  |\n"
        "1 | x = 0x\n"
        "  |     ^^\n"
        "invalid integer '0x'"
    )

    # missing value
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads(".")
    msg = str(exc.value)
    assert "line 1, column" in msg
    assert "missing value" in msg

    # multiple newlines
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("\n\nval=.")
    msg = str(exc.value)
    assert "line 3, column 5" in msg
    assert "invalid mantissa" in msg

    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("\n\n.")
    msg = str(exc.value)
    assert "line 3, column" in msg
    assert "missing value" in msg


def test_missing_value():
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("\n\nfwfw=")
    msg = str(exc.value)
    assert "line 3, column 6" in msg
    assert "string values must be quoted" in msg


def test_invalid_char_quotes():
    with pytest.raises(tomllib.TOMLDecodeError) as exc:
        tomllib.loads("v = '\n'")
    assert "key with no value, expected `=`" in str(exc.value)


def test_type_error():
    with pytest.raises(TypeError) as exc:
        tomllib.loads(b"v = 1")  # type: ignore[arg-type]
    assert str(exc.value) in (
        "Expected str object, not 'bytes'", "str object expected; got bytes",
    )

    with pytest.raises(TypeError) as exc:
        tomllib.loads(False)  # type: ignore[arg-type]  # noqa: FBT003
    assert str(exc.value) in (
        "Expected str object, not 'bool'", "str object expected; got bool",
    )


def test_invalid_parse_float():
    def dict_returner(s: str) -> dict[Any, Any]:
        return {}

    def list_returner(s: str) -> list[Any]:
        return []

    err_msg = "parse_float must not return dicts or lists"

    for invalid_parse_float in (dict_returner, list_returner):
        with pytest.raises(ValueError, match=err_msg) as exc:
            tomllib.loads("f=0.1", parse_float=invalid_parse_float)
        assert str(exc.value) == err_msg


def test_tomldecodeerror_attributes():
    data = """\
title = "TOML Example"

[owner]
name = "Tom Preston-Werner"
dob = 1979-05-27T07:32:00-08:00

x =
"""
    with pytest.raises(tomllib.TOMLDecodeError) as exc_info:
        tomllib.loads(data)

    exc = exc_info.value

    assert exc.msg.startswith("TOML parse error")
    assert exc.doc == data
    assert exc.pos == 96
    assert exc.lineno == 7
    assert exc.colno == 4
    assert f"line {exc.lineno}, column {exc.colno}" in str(exc)


def test_unsupported_version():
    with pytest.raises(
            ValueError,
            match="Unsupported TOML version",
    ):
        tomllib.loads("x = 1", toml_version="2")  # ty: ignore[invalid-argument-type]



import platform
import sys
import types
from decimal import Decimal
from pathlib import Path

import pytest
import toml_rs

from ._types import ParseFloat
from .burntsushi import convert, normalize
from .helpers import TOML
from .test_data import VALID_PAIRS_1_0_0 as VALID_PAIRS

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib  # ty: ignore


def test_example_toml() -> None:
    toml_str = TOML.read_text(encoding="utf-8")
    assert tomllib.loads(toml_str) == toml_rs.loads(toml_str)


@pytest.mark.parametrize("lib", [tomllib, toml_rs])
def test_text_mode_typeerror(lib: types.ModuleType) -> None:
    err_msg = "File must be opened in binary mode, e.g. use `open('foo.toml', 'rb')`"
    with (
        Path(TOML).open(encoding="utf-8") as f,
        pytest.raises(TypeError) as exc,
    ):
        lib.load(f)
    assert err_msg in str(exc.value)


@pytest.mark.parametrize(
    ("valid", "expected"),
    VALID_PAIRS,
    ids=[p[0].stem for p in VALID_PAIRS],
)
def test_tomllib_vs_tomlrs(valid: Path, expected: Path) -> None:
    toml_str = valid.read_bytes().decode("utf-8")
    try:
        toml_str.encode("ascii")
    except UnicodeEncodeError:
        pytest.skip(f"Skipping Unicode content test: {valid.name}")

    tomllib_ = normalize(convert(tomllib.loads(toml_str)))
    toml_rs_ = normalize(convert(toml_rs.loads(toml_str)))

    assert tomllib_ == toml_rs_, f"Mismatch between tomllib and toml_rs for {valid.name}"


@pytest.mark.skipif(
    platform.python_implementation() == "PyPy",
    reason="PyPy's `Decimal` parsing hits the int string "
           "conversion digit limit for very large numbers.",
)
@pytest.mark.parametrize(
    "parse_float",
    [float, Decimal],
    ids=["float", "Decimal"],
)
def test_parse_float(parse_float: ParseFloat) -> None:
    num = "9" * 47
    f = f"{num}.{num}"
    t = f"x = {f}"

    tomllib_ = tomllib.loads(t, parse_float=parse_float)
    toml_rs_1 = toml_rs.loads(t, toml_version="1.0.0", parse_float=parse_float)
    toml_rs_1_1 = toml_rs.loads(t, toml_version="1.1.0", parse_float=parse_float)

    assert tomllib_ == toml_rs_1
    assert tomllib_ == toml_rs_1_1
